(******************************************************************************)
(* Diese Unit, stellt eine einfach zu nutztende Schrift für OpenGL zur        *)
(* Verfügung. Der Schriftsatz beruht auf dem Original DOS Schriftsatz         *)
(* in der Größe 8x12.                                                         *)
(*                                                                            *)
(* Benutzung : in OpenGLControl1MakeCurrent oder später einmalig              *)
(*             Create_ASCII_Font aufrufen. Danach steht                       *)
(*                                                                            *)
(*             OpenGL_ASCII_Font zur Verfügung.                               *)
(*                                                                            *)
(* Support : www.Corpsman.de                                                  *)
(*                                                                            *)
(* License : This Unit is Freeware, for non commercial use only.              *)
(*                                                                            *)
(* Warranty : There is NO Warranty.                                           *)
(*                                                                            *)
(******************************************************************************)
Unit uOpenGL_ASCII_Font;

Interface

Uses
  IntfGraphics, fpImage, Graphics, LCLType, dglopengl, LResources, uvectormath,
  math;

Type

  { TOpenGL_ASCII_Font }

  TOpenGL_ASCII_Font = Class
  private
    FBaseList: gluint; // Basispointer, für die OpenGL List
    FCharcount: integer; // Anzahl der Zeichen, sollte immer 256 sein !!
    fColor: TVector3; // Farbe der Schrift
    fsize: Single; // Größe der Schrift in Pixeln ( Höhe )
    fCharheight: integer; // "Echte" Größe eines Buchstabens in Pixeln
    fCharwidth: integer; // "Echte" Größe eines Buchstabens in Pixeln
    Function getcolor: TColor;
    Function getcolor3v: TVector3;
    Function getcolorfp: TFPColor;
    Function getsize: Single;
    Procedure RenderChar(Number: integer); // Zeichnet ein Zeichen
    Procedure setColor(AValue: TColor);
    Procedure setColor3v(AValue: TVector3);
    Procedure setColorfp(AValue: TFPColor);
    Procedure setsize(AValue: Single);
  public
    Property Color: TColor read getcolor write setColor; // Zugriff auf die Schriftfarbe
    Property Color3v: TVector3 read getcolor3v write setColor3v; // Zugriff auf die Schriftfarbe
    Property Colorfp: TFPColor read getcolorfp write setColorfp; // Zugriff auf die Schriftfarbe
    Property Size: Single read getsize write setsize; // Höhe der Schrift in Pixeln
    Constructor Create(Const Bitmap: Tbitmap; CharWidth, CharHeight, CharCount: Integer); // Alles was Weiß ist, ist sichtbar, der Rest Transparent
    Destructor destroy; override;
    Procedure Textout(x, y: Integer; Text: String); // Zeichnet einen Text, unter Berücksichtigung von CRT's
    Function TextWidth(Text: String): single; // Breite des Textes in Pixeln
    Function TextHeight(text: String): single; // Höhe des Textes in Pixeln
  End;

Var
  OpenGL_ASCII_Font: TOpenGL_ASCII_Font = Nil;

Procedure Create_ASCII_Font(); // Muss in Make Current aufgerufen werden, da es OpenGLBefehle nutzt.

// Switcher zwischen 2D und 3D-Modus
Procedure Go2d(Width_2D, Height_2d: Integer);
Procedure Exit2d();

Implementation

Procedure Go2d(Width_2D, Height_2d: Integer);
Begin
  glMatrixMode(GL_PROJECTION);
  glPushMatrix(); // Store The Projection Matrix
  glLoadIdentity(); // Reset The Projection Matrix
  glOrtho(0, Width_2D, Height_2d, 0, -1, 1); // Set Up An Ortho Screen
  glMatrixMode(GL_MODELVIEW);
  glPushMatrix(); // Store old Modelview Matrix
  glLoadIdentity(); // Reset The Modelview Matrix
End;

Procedure Exit2d();
Begin
  glMatrixMode(GL_PROJECTION);
  glPopMatrix(); // Restore old Projection Matrix
  glMatrixMode(GL_MODELVIEW);
  glPopMatrix(); // Restore old Projection Matrix
End;

Procedure Create_ASCII_Font();
Var
  bitmap: TBitmap;
Begin
  If Assigned(OpenGL_ASCII_Font) Then OpenGL_ASCII_Font.free;
  Bitmap := TBitmap.Create;
  bitmap.LoadFromLazarusResource('OpenGLFont');
  OpenGL_ASCII_Font := TOpenGL_ASCII_Font.Create(bitmap, 8, 12, 256);
  bitmap.Free;
End;

Function FPColorToColor(Const Color: TFPColor): TColor;
Begin
  result := byte(color.red Shr 8) Or (color.green And $FF00) Or ((color.blue And $FF00) Shl 8);
End;

Function ColorToFPColor(Const Color: TColor): TFPColor;
Begin
  result.alpha := 0;
  result.red := (byte(Color) Shl 8) Or $FF;
  result.green := ((Color) And $FF00) Or $FF;
  result.blue := (((Color) And $FF0000) Shr 8) Or $FF;
End;

{ TOpenGLFont }

Constructor TOpenGL_ASCII_Font.Create(Const Bitmap: Tbitmap; charwidth, CharHeight,
  Charcount: Integer);
Var
  cxc, ax, ay, i, j: Integer;
  k: GLuint;
  TempIntfImg: TLazIntfImage;
  acol: TColor;
Begin
  Inherited create;
  fsize := CharHeight;
  fCharheight := CharHeight;
  fCharwidth := charwidth;
  fColor := v3(1, 1, 1);
  FCharcount := Charcount;
  FBaseList := glGenLists(Charcount);
  TempIntfImg := TLazIntfImage.Create(0, 0);
  TempIntfImg.LoadFromBitmap(Bitmap.Handle, Bitmap.MaskHandle);
  cxc := Bitmap.width Div Charwidth;
  For k := 0 To FCharcount - 1 Do Begin
    glNewList(FBaseList + k, GL_COMPILE); // Start Building A List
    glbegin(GL_POINTS);
{$WARNINGS off}
    ax := (k Mod cxc) * CharWidth;
    ay := (k Div cxc) * Charheight;
{$WARNINGS on}
    // Auslesen der Einzelnen Chars
    For i := 0 To charwidth - 1 Do
      For j := 0 To charheight - 1 Do Begin
        If (ax + i < TempIntfImg.Width) And (ay + j < TempIntfImg.Height) Then Begin
          acol := FPColorToColor(TempIntfImg.Colors[ax + i, ay + j]);
          If acol = clwhite Then
            glVertex2i(i, j);
        End;
      End;
    glend;
    gltranslated(CharWidth, 0, 0);
    glEndList(); // Done Building The Display List
  End;
  TempIntfImg.free;
End;

Destructor TOpenGL_ASCII_Font.destroy;
Begin
  glDeleteLists(FBaseList, FCharcount); // Delete All Display Lists
End;

Procedure TOpenGL_ASCII_Font.Textout(x, y: Integer; Text: String);
Var
  k: integer;
  c: integer;
  sc: Single;
Begin
  glPushMatrix();
  glTranslatef(x, y, 0);
  glColor3fv(@fColor);
  sc := fsize / fCharheight;
  glPointSize(sc + 0.45); // Ohne diesen Faktor, sind in diversen Skallierungen keine ganzen Buchstaben zu sehen.
  glScalef(sc, sc, sc);
  glPushMatrix();
  For k := 1 To length(text) Do Begin
    c := ord(text[k]);
    If c = 13 Then Begin
      glPopMatrix();
      glTranslatef(0, fCharheight, 0);
      glPushMatrix();
    End
    Else If c <> 10 Then Begin
      RenderChar(c);
    End;
  End;
  glPopMatrix();
  glPopMatrix();
End;

Function TOpenGL_ASCII_Font.Textwidth(Text: String): single;
Var
  j: integer;
Begin
  j := 0;
  While pos(#13, text) <> 0 Do Begin
    j := max(j, pos(#13, text));
    delete(Text, 1, pos(#13, text));
  End;
  j := max(j, length(Text));
  result := j * (fsize / fCharheight) * fCharwidth;
End;

Function TOpenGL_ASCII_Font.Textheight(text: String): single;
Var
  i, j: integer;
Begin
  j := 1;
  For i := 1 To length(text) Do Begin
    If ord(text[i]) = 13 Then inc(j);
  End;
  result := j * fsize;
End;

Procedure TOpenGL_ASCII_Font.RenderChar(Number: integer);
Begin
  glListBase(FBaseList);
  glCallLists(1, GL_UNSIGNED_BYTE, @Number);
End;

Procedure TOpenGL_ASCII_Font.setColor(AValue: TColor);
Begin
  fcolor.x := byte(Avalue) / 255;
  fcolor.y := byte(Avalue Shr 8) / 255;
  fcolor.z := byte(Avalue Shr 16) / 255;
End;

Procedure TOpenGL_ASCII_Font.setColor3v(AValue: TVector3);
Begin
  fColor := AValue;
End;

Procedure TOpenGL_ASCII_Font.setColorfp(AValue: TFPColor);
Begin
  fcolor.x := ((AValue.red And $FF00) Shr 8) / 255;
  fcolor.y := ((AValue.green And $FF00) Shr 8) / 255;
  fcolor.z := ((AValue.blue And $FF00) Shr 8) / 255;
End;

Function TOpenGL_ASCII_Font.getsize: Single;
Begin
  result := fsize;
End;

Function TOpenGL_ASCII_Font.getcolor: TColor;
Var
  r, g, b: integer;
Begin
  r := max(0, min(255, round(fColor.x * 255)));
  g := max(0, min(255, round(fColor.y * 255)));
  b := max(0, min(255, round(fColor.z * 255)));
  result := r Or (g Shl 8) Or (b Shl 16);
End;

Function TOpenGL_ASCII_Font.getcolor3v: TVector3;
Begin
  result := fColor;
End;

Function TOpenGL_ASCII_Font.getcolorfp: TFPColor;
Var
  r, g, b: integer;
Begin
  r := max(0, min(255, round(fColor.x * 255))) Shl 8 Or $FF;
  g := max(0, min(255, round(fColor.y * 255))) Shl 8 Or $FF;
  b := max(0, min(255, round(fColor.z * 255))) Shl 8 Or $FF;
  result.red := r;
  result.green := g;
  result.blue := b;
  result.alpha := 0;
End;

Procedure TOpenGL_ASCII_Font.setsize(AValue: Single);
Begin
  fsize := AValue;
End;

Initialization

{$I uopenGL_ascii_font.ressource}

Finalization

  If assigned(OpenGL_ASCII_Font) Then OpenGL_ASCII_Font.free;
  OpenGL_ASCII_Font := Nil;

End.

