(******************************************************************************)
(* Autor       : Corpsman                                          16.07.2010 *)
(*                                                                            *)
(* Support     : www.Corpsman.de                                              *)
(*                                                                            *)
(* Description : This unit holds a lot of vector algorithms, often needed in  *)
(*               programms.                                                   *)
(*               The vector and matrix implementation is byte compatible      *)
(*               with the OpenGL standard if used single as TBasetype.        *)
(*                                                                            *)
(* License     : This unit is freeware, for non commercial use only.          *)
(*                                                                            *)
(* Warranty    : There is no warranty, neither in correctness of the          *)
(*               implementation, nor anything other that could happen         *)
(*               or go wrong.                                                 *)
(*                                                                            *)
(* Version     : 0.01                                                         *)
(*                                                                            *)
(******************************************************************************)
Unit uvectormath;

{$MODE objfpc}{$H+}

(*
 * Mit diesem Schalter kann das Überladen der Standart Operatoren Aktiviert
 * werden.
 *)
{$DEFINE UseOperandOverloading}

Interface

Uses
  Math // Sin Cos degtorad radtodeg
  , sysutils // exception
  ;

Type

  TBaseType = Single; // Alle Komponenten bestehen aus BaseType, zur Nutzung von OpenGL ist Single zwingend !!

  TVector2 = Record
    x, y: TBaseType;
  End;

  TVector3 = Record
    x, y, z: TBaseType;
  End;

  TVector4 = Record
    x, y, z, w: TBaseType;
  End;

  TVectorN = Array Of TBaseType; // Allgemein

  TVector2Array = Array Of TVector2;

  TVector3Array = Array Of TVector3;

  TVector4Array = Array Of TVector4;

  TVectorNArray = Array Of TVectorN;

  TMatrix2x2 = Array[0..1, 0..1] Of TBaseType;

  TMatrix3x3 = Array[0..2, 0..2] Of TBaseType;

  TMatrix4x4 = Array[0..3, 0..3] Of TBaseType;

  TMatrixNxM = Array Of Array Of TBaseType; // Allgemein

{$IFDEF UseOperandOverloading}
  Operator + (a, b: TVector2): TVector2;
  Operator - (a, b: TVector2): TVector2;
  Operator * (a, b: TVector2): TVector2;
  Operator * (a, b: TVector2): TBaseType;
  Operator * (s: TBaseType; v: TVector2): TVector2;
  Operator * (v: TVector2; s: TBaseType): TVector2;


  Operator + (a, b: TVector3): TVector3;
  Operator - (a, b: TVector3): TVector3;
  Operator * (a, b: TVector3): TVector3;
  Operator * (a, b: TVector3): TBaseType;
  Operator * (s: TBaseType; v: TVector3): TVector3;
  Operator * (v: TVector3; s: TBaseType): TVector3;

  Operator * (m: TMatrix3x3; v: TVector3): Tvector3;
  Operator * (m: TMatrix4x4; v: TVector4): Tvector4;

{$ENDIF}

Function mod2(a, b: Integer): integer; // Bildet Negative "a" auf die passenden positiven Restklassen von b ab.
Function RandomRange(a, b: integer): integer; // Erzeugt eine Zufallszahl im Bereich [a..b)

// Konvertierungsfunktionen
Function V2(Const X, Y: TBaseType): TVector2;
Function V3(Const X, Y, Z: TBaseType): TVector3;
Function V4(Const X, Y, Z, W: TBaseType): TVector4; Overload;
Function V4(Const V: Tvector3; Const W: TBaseType): TVector4; Overload;
Function M3x3(x11, x12, x13, x21, x22, x23, x31, x32, x33: TBaseType): TMatrix3x3;

Function abs(Const V: TVector2): TBaseType; overload;
Function abs(Const V: TVector3): TBaseType; overload;
Function abs(Const V: TVector4): TBaseType; overload;

// Grundrechenarten
Function AddV2(Const V1, V2: TVector2): Tvector2;
Function AddV3(Const V1, V2: TVector3): Tvector3;
Function AddV4(Const V1, V2: TVector4): Tvector4;

Function SubV2(Const V1, V2: TVector2): Tvector2;
Function SubV3(Const V1, V2: TVector3): Tvector3;
Function SubV4(Const V1, V2: TVector4): Tvector4;

Function LenV2(Const V: TVector2): TBaseType;
Function LenV3(Const V: TVector3): TBaseType;
Function LenV4(Const V: TVector4): TBaseType;

Function NormV2(Const Value: Tvector2): Tvector2;
Function NormV3(Const Value: Tvector3): Tvector3;

Function ScaleV2(Const Scalar: TBaseType; Const V: Tvector2): Tvector2;
Function ScaleV3(Const Scalar: TBaseType; Const V: Tvector3): Tvector3;
Function ScaleV4(Const Scalar: TBaseType; Const V: Tvector4): Tvector4;

Function DotV2(Const V1, V2: TVector2): TBaseType;
Function DotV3(Const V1, V2: TVector3): TBaseType;

Function AngleV2(Const V1, V2: TVector2): TBaseType; // berechnet den Winkel zwischen 2 Vektoren, ACHTUNG, AngleV2 ist immer in [0..180], der Drehsinn wird Ignoriert

Function CrossV2(Const a, b: Tvector2): Tvector2;
Function CrossV3(Const V1, V2: Tvector3): Tvector3;

Function CompareV2(Const V1, V2: TVector2; Const Toleranze: TBaseType): Boolean; // Schaut ob 2 Vektoren Gleich sind

// Matrizen Operatoren
Function IdentityMatrix3x3: TMatrix3x3;
Function IdentityMatrix4x4: TMatrix4x4;

Function AddMatrix3x3(M1, M2: TMatrix3x3): TMatrix3x3;

Function InvertMatrix(M: TMatrix3x3): TMatrix3x3; overload; // Mittels Gauß-Jordan-Algorithmus
Function InvertMatrix(M: TMatrix4x4): TMatrix4x4; overload; // Mittels Gauß-Jordan-Algorithmus
Function InvertMatrix2(M: TMatrix3x3): TMatrix3x3; // Berechnung über die Adjunkte

Function TransposeMatrix(Const M: TMatrix3x3): TMatrix3x3; overload;
Function TransposeMatrix(Const M: TMatrix4x4): TMatrix4x4; overload;

Function MulVectorMatrix(Const V: TVector3; Const M: TMatrix3x3): TVector3; overload;
Function MulVectorMatrix(Const V: TVector4; Const M: TMatrix4x4): TVector4; overload;

Function MulMatrix(Const M1, M2: TMatrix3x3): TMatrix3x3; overload;
Function MulMatrix(Const M1, M2: TMatrix4x4): TMatrix4x4; overload;

// Erzeugt eine Rotationsmatrix aus Angle und RotationVector
// Jeder Vektor, welcher mit dieser Matrix Multipliziert wird, wird um die in
// RotationVector angegebenen Achsen Rotiert ( siehe gLRotate )
Function GenerateRotationMatrix(Angle: TBaseType; RotationVector: TVector3): TMatrix4x4;

// True wenn das Objekt das durch die Punkte Angegeben ist ein Konvexes Polygon
// Wichtig, die Reihenfolge der Punkte muss korrekt sein
// Der Drehsinn des Polygons wird automatisch ermittelt
Function PointArrIsConvexPolygon(Const Points: TVector2Array): Boolean;

// Berechnet aus einer Punktwolke die Convexe Hülle und gibt diese zurück
Function PointsToConvexHull(Const Points: TVector2Array): TVector2Array;

// Berechnet den Flächeninhalt eines Polygones
Function CalulatePolygonArea(Const Points: TVector2Array): TBaseType;

// Berechnet die Augmatrix eines Kamerasystems
Function CalculateOrthoMatrix(EyePos, Dir, Up: TVector3): TMatrix4x4;

// Setzt die Shadow Matrix, derart, dass sie in OpenGL dazu benutzt werden kann, aus Sicht einer Lichtquelle "Schatten" werfende
// Objekte in den Stencil Buffer zu zeichnen.
// ShadowMatrix = Ergebniss
// Normale = Normale des Schatten Empfangenden Objects ( mus eine gerade Fläche sein
// Point = Ein Punkt der Schatten Empfandenden Fläche
// Koordinaten der Lichtquelle ( Im Zweifel ist die W-Komponente auf 1 zu setzen )!!
Procedure GenerateShadowMatrix(Out ShadowMatrix: TMatrix4x4; Const Normal, Point: TVector3; Light: TVector4);

// Errechnet den Winkel Zwischen (X,0) und (X,Y) im Gradmaß aus
Function ArcTangens(Const X, Y: Extended): Extended;

// Prüft ob P im Achsenparalellen Rechteck TL, BR ist
Function PointInRect(Const P, TL, BR: TVector2): Boolean;

// Prüft ob P im Achsenparalellen Quader TL, BR ist
Function PointInCube(Const P, TL, BR: Tvector3): Boolean;

// Die Punkte P1 - P4 Spannen ein Object Orientiertes Rechetck auf
// Die Punkte müssen im Gegenuhrzeigersinn übergeben werden.
// Result = True wenn P innerhalb dieses Reckeckes liegt
Function PointInOOBB(Const P, P1, P2, P3, P4: TVector2): Boolean;

// True wenn sich die Geradenstücke AB und CD Schneiden
Function IntersectLine(Const A, B, C, D: TVector2): Boolean;

// True wenn sich zwei OOBB's überschneiden
Function IntersectOOBB(Const A, B, C, D, AA, BB, CC, DD: TVector2): Boolean;

// Berechnet das Trägheitsmoment eines Körpers bezüglich einer Geraden die durch den Nullpunkt geht und den Richtungsvektor v hat.
// Entnommen aus dem Bronstein ( 5. Auflage erschienen 2001 ) Formel  4.70b
// Für genauere Deteils bitte dort Nachlesen
Function InertialTensor(v: TVector3; m: Single): TMatrix3x3;

// Berechnet aus 3 gegebenen Punkten und einem Punkt "unterhalb" der aufgespannten
// Ebene einen zugehörigen Normalenvektor, welcher nach außen zeigt.
Function GetNormal(EbenenPunkt1, EbenenPunkt2, EbenenPunkt3, Punktinnen: Tvector3): Tvector3;

// Debugg Operationen
Function Print(Const V: Tvector2): String; overload;
Function Print(Const V: Tvector3): String; overload;
Function Print(Const V: Tvector4): String; overload;
Function Print(Const M: TMatrix3x3): String; overload;
Function Print(Const M: TMatrix4x4): String; overload;

Implementation

{$IFDEF UseOperandOverloading}

Operator + (a, b: TVector2): TVector2;
Begin
  result := addv2(a, b);
End;

Operator - (a, b: TVector2): TVector2;
Begin
  result := subv2(a, b);
End;

Operator * (a, b: TVector2): TVector2;
Begin
  result := CrossV2(a, b);
End;

Operator * (s: TBaseType; v: TVector2): TVector2;
Begin
  result := ScaleV2(s, v);
End;

Operator * (a, b: TVector2): TBaseType;
Begin
  result := DotV2(a, b);
End;

Operator * (v: TVector2; s: TBaseType): TVector2;
Begin
  result := ScaleV2(s, v);
End;

Operator + (a, b: TVector3): TVector3;
Begin
  result := addv3(a, b);
End;

Operator - (a, b: TVector3): TVector3;
Begin
  result := subv3(a, b);
End;

Operator * (a, b: TVector3): TVector3;
Begin
  result := CrossV3(a, b);
End;

Operator * (a, b: TVector3): TBaseType;
Begin
  result := DotV3(a, b);
End;

Operator * (s: TBaseType; v: TVector3): TVector3;
Begin
  result := ScaleV3(s, v);
End;

Operator * (v: TVector3; s: TBaseType): TVector3;
Begin
  result := ScaleV3(s, v);
End;

Operator * (m: TMatrix3x3; v: TVector3): Tvector3;
Begin
  result := MulVectorMatrix(v, m);
End;

Operator * (m: TMatrix4x4; v: TVector4): Tvector4;
Begin
  result := MulVectorMatrix(v, m);
End;

{$ENDIF}

Function mod2(a, b: Integer): integer;
Begin
  If a >= 0 Then
    result := a Mod b
  Else Begin
    result := b - ((-a) Mod b);
  End;
End;

Function RandomRange(a, b: integer): integer;
Var
  i: integer;
Begin
  If a > b Then Begin
    i := a;
    a := b;
    b := i;
  End;
  i := b - a;
  result := random(i) + a;
End;

Function V2(Const x, y: TBaseType): TVector2;
Begin
  result.x := x;
  result.y := y;
End;

Function V3(Const x, y, z: TBaseType): TVector3;
Begin
  result.x := x;
  result.y := y;
  result.z := z;
End;

Function V4(Const x, y, z, w: TBaseType): TVector4; overload;
Begin
  result.x := x;
  result.y := y;
  result.z := z;
  result.w := w;
End;

Function V4(Const v: Tvector3; Const w: TBaseType): TVector4; overload;
Begin
  result.x := v.x;
  result.y := v.y;
  result.z := v.z;
  result.w := w;
End;

Function abs(Const V: TVector2): TBaseType;
Begin
  result := LenV2(v);
End;

Function abs(Const V: TVector3): TBaseType;
Begin
  result := LenV3(v);
End;

Function abs(Const V: TVector4): TBaseType;
Begin
  result := LenV4(v);
End;

Function M3x3(x11, x12, x13, x21, x22, x23, x31, x32, x33: TBaseType): TMatrix3x3;
Begin
  Result[0, 0] := x11;
  Result[0, 1] := x12;
  Result[0, 2] := x13;
  Result[1, 0] := x21;
  Result[1, 1] := x22;
  Result[1, 2] := x23;
  Result[2, 0] := x31;
  Result[2, 1] := x32;
  Result[2, 2] := x33;
End;

Function AddV2(Const v1, v2: TVector2): Tvector2;
Begin
  result.x := v1.x + v2.x;
  result.y := v1.y + v2.y;
End;

Function AddV3(Const v1, v2: TVector3): Tvector3;
Begin
  result.x := v1.x + v2.x;
  result.y := v1.y + v2.y;
  result.z := v1.z + v2.z;
End;

Function AddV4(Const v1, v2: TVector4): Tvector4;
Begin
  result.x := v1.x + v2.x;
  result.y := v1.y + v2.y;
  result.z := v1.z + v2.z;
  result.w := v1.w + v2.w;
End;

Function SubV2(Const v1, v2: TVector2): Tvector2;
Begin
  result.x := v1.x - v2.x;
  result.y := v1.y - v2.y;
End;

Function SubV3(Const v1, v2: TVector3): Tvector3;
Begin
  result.x := v1.x - v2.x;
  result.y := v1.y - v2.y;
  result.z := v1.z - v2.z;
End;

Function SubV4(Const v1, v2: TVector4): Tvector4;
Begin
  result.x := v1.x - v2.x;
  result.y := v1.y - v2.y;
  result.z := v1.z - v2.z;
  result.w := v1.w - v2.w;
End;

Function LenV2(Const V: TVector2): TBaseType;
Begin
  result := sqrt(sqr(v.x) + sqr(v.y));
End;

Function lenV3(Const V: TVector3): TBaseType;
Begin
  result := sqrt(sqr(v.x) + sqr(v.y) + sqr(v.z));
End;

Function lenV4(Const V: TVector4): TBaseType;
Begin
  result := sqrt(sqr(v.x) + sqr(v.y) + sqr(v.z) + sqr(v.w));
End;

Function NormV2(Const Value: Tvector2): Tvector2;
Var
  betrag: TBaseType;
Begin
  betrag := lenV2(value);
  If Betrag <> 0 Then
    result := ScaleV2(1 / Betrag, Value)
  Else
    result := v2(0.0, 0.0);
End;

Function NormV3(Const Value: Tvector3): Tvector3;
Var
  betrag: TBaseType;
Begin
  betrag := lenV3(value);
  If Betrag <> 0 Then
    result := ScaleV3(1 / Betrag, Value)
  Else
    result := v3(0.0, 0.0, 0.0);
End;

Function ScaleV2(Const Scalar: TBaseType; Const v: Tvector2): Tvector2;
Begin
  result.x := scalar * v.x;
  result.y := scalar * v.y;
End;

Function ScaleV3(Const Scalar: TBaseType; Const v: Tvector3): Tvector3;
Begin
  result.x := scalar * v.x;
  result.y := scalar * v.y;
  result.z := scalar * v.z;
End;

Function ScaleV4(Const Scalar: TBaseType; Const v: Tvector4): Tvector4;
Begin
  result.x := scalar * v.x;
  result.y := scalar * v.y;
  result.z := scalar * v.z;
  result.w := scalar * v.w;
End;

Function DotV2(Const v1, v2: TVector2): TBaseType;
Begin
  result := v1.x * v2.x + v1.y * v2.y;
End;

Function DotV3(Const v1, v2: TVector3): TBaseType;
Begin
  result := v1.x * v2.x + v1.y * v2.y + v1.z * v2.z;
End;

Function AngleV2(Const v1, v2: TVector2): TBasetype;
Var
  w, z, n: TBasetype;
Begin
  n := abs(lenv2(v1) * lenv2(v2));
  If n <> 0 Then Begin
    z := Dotv2(v1, v2);
    w := z / n;
    result := radtodeg(arccos(w));
  End
  Else
    result := 0;
End;

Function CrossV2(Const a, b: Tvector2): Tvector2;
Begin
  result := v2(
    a.y * b.x - a.x * b.y,
    a.x * b.y - a.y * b.x
    );
End;

Function CrossV3(Const v1, v2: Tvector3): Tvector3;
Begin
  result := v3(
    v1.y * v2.z - v1.z * v2.y,
    v1.z * v2.x - v1.x * v2.z,
    v1.x * v2.y - v1.y * v2.x
    );
End;

Function Comparev2(Const v1, v2: TVector2; Const Toleranze: TBaseType): Boolean;
Var
  d: TVector2;
Begin
  d := subv2(v1, v2);
  result := lenv2(d) <= Toleranze;
End;

Function IdentityMatrix3x3: TMatrix3x3;
Begin
  result[0, 0] := 1;
  result[1, 0] := 0;
  result[2, 0] := 0;
  result[0, 1] := 0;
  result[1, 1] := 1;
  result[2, 1] := 0;
  result[0, 2] := 0;
  result[1, 2] := 0;
  result[2, 2] := 1;
End;

Function IdentityMatrix4x4: TMatrix4x4;
Begin
  result[0, 0] := 1;
  result[1, 0] := 0;
  result[2, 0] := 0;
  result[3, 0] := 0;
  result[0, 1] := 0;
  result[1, 1] := 1;
  result[2, 1] := 0;
  result[3, 1] := 0;
  result[0, 2] := 0;
  result[1, 2] := 0;
  result[2, 2] := 1;
  result[3, 2] := 0;
  result[0, 3] := 0;
  result[1, 3] := 0;
  result[2, 3] := 0;
  result[3, 3] := 1;
End;

Function AddMatrix3x3(M1, M2: TMatrix3x3): TMatrix3x3;
Var
  i, j: Integer;
Begin
  For i := 0 To 2 Do
    For j := 0 To 2 Do
      Result[i, j] := m1[i, j] + m2[i, j];
End;

(*

Berechnet zu
    ( x x x )
M = ( x x x )
    ( x x x )

die Matrix M' so das gilt

M * M' = 1

Wenn dies nicht möglich ist, weil z.b. die Determinante = 0, wird eine Exception geworfen.

*)

Function InvertMatrix(M: TMatrix3x3): TMatrix3x3;
Const
  dim = 3 - 1;
  (*
  Addiert auf Zeile x die Zeile y
  *)
  Procedure Line_X_Add_Line_Y(x, y: Integer);
  Var
    i: Integer;
  Begin
    For i := 0 To dim Do Begin
      result[i, x] := result[i, x] + result[i, y];
      m[i, x] := m[i, x] + m[i, y];
    End;
  End;

  (*
  Zieht von Zeile x die Zeile y ab.
  *)
  Procedure Line_X_Sub_V_Mul_Line_Y(x: integer; v: TBaseType; y: Integer);
  Var
    i: Integer;
  Begin
    For i := 0 To dim Do Begin
      result[i, x] := result[i, x] - v * result[i, y];
      m[i, x] := m[i, x] - v * m[i, y];
    End;
  End;

  (*
  Multipliziert die Zeile Line mit dem Wert Value durch
  *)
  Procedure Mul_Line(Value: TBaseType; Line: Integer);
  Var
    i: Integer;
  Begin
    For i := 0 To dim Do Begin
      result[i, Line] := result[i, Line] * Value;
      m[i, Line] := m[i, Line] * Value;
    End;
  End;

Var
  i, j: Integer;
  b: Boolean;
Begin
  // Init Ergebniss mit 1 Matrix
  For i := 0 To dim Do
    For j := 0 To dim Do
      If i = j Then
        result[i, j] := 1
      Else
        Result[i, j] := 0;
  // Wir Lösen Spaltenweise
  For i := 0 To dim Do Begin
    // Auf den Diagonal Elementen Erzeugen wir die 1, also darf hier keine 0 stehen.
    If M[i, i] = 0 Then Begin
      b := false;
      For j := i + 1 To dim Do Begin
        If m[i, j] <> 0 Then Begin
          Line_X_Add_Line_Y(i, j);
          b := True;
          break;
        End;
      End;
      If Not b Then Begin
        Raise Exception.create('"uvectormath.InvertMatrix" Error unable to invert the Matrix.');
      End;
    End;
    // Nun Lösen wir die Spalte
    // Zuerst erzeugen wir die 1
    Mul_Line(1 / m[i, i], i);
    // Dann Erzeugen wir die 0 en
    For j := 0 To dim Do
      If i <> j Then
        Line_X_Sub_V_Mul_Line_Y(j, m[i, j], i);
  End;
End;

(*

Berechnet zu
    ( x x x x )
    ( x x x x )
M = ( x x x x )
    ( x x x x )

die Matrix M' so das gilt

M * M' = 1

Wenn dies nicht möglich ist, weil z.b. die Determinante = 0, wird eine Exception geworfen.

*)

Function InvertMatrix(M: TMatrix4x4): TMatrix4x4;
Const
  Dim = 4 - 1;
  (*
  Addiert auf Zeile x die Zeile y
  *)
  Procedure Line_X_Add_Line_Y(x, y: Integer);
  Var
    i: Integer;
  Begin
    For i := 0 To Dim Do Begin
      result[i, x] := result[i, x] + result[i, y];
      m[i, x] := m[i, x] + m[i, y];
    End;
  End;

  (*
  Zieht von Zeile x die Zeile y ab.
  *)
  Procedure Line_X_Sub_V_Mul_Line_Y(x: integer; v: TBaseType; y: Integer);
  Var
    i: Integer;
  Begin
    For i := 0 To Dim Do Begin
      result[i, x] := result[i, x] - v * result[i, y];
      m[i, x] := m[i, x] - v * m[i, y];
    End;
  End;

  (*
  Multipliziert die Zeile Line mit dem Wert Value durch
  *)
  Procedure Mul_Line(Value: TBaseType; Line: Integer);
  Var
    i: Integer;
  Begin
    For i := 0 To Dim Do Begin
      result[i, Line] := result[i, Line] * Value;
      m[i, Line] := m[i, Line] * Value;
    End;
  End;

Var
  i, j: Integer;
  b: Boolean;
Begin
  // Init Ergebniss mit 1 Matrix
  For i := 0 To Dim Do
    For j := 0 To Dim Do
      If i = j Then
        result[i, j] := 1
      Else
        Result[i, j] := 0;
  // Wir Lösen Spaltenweise
  For i := 0 To Dim Do Begin
    // Auf den Diagonal Elementen Erzeugen wir die 1, also darf hier keine 0 stehen.
    If M[i, i] = 0 Then Begin
      b := false;
      For j := i + 1 To Dim Do Begin
        If m[i, j] <> 0 Then Begin
          Line_X_Add_Line_Y(i, j);
          b := True;
          break;
        End;
      End;
      If Not b Then Begin
        Raise Exception.create('"uvectormath.InvertMatrix" Error unable to invert the Matrix.');
      End;
    End;
    // Nun Lösen wir die Spalte
    // Zuerst erzeugen wir die 1
    Mul_Line(1 / m[i, i], i);
    // Dann Erzeugen wir die 0 en
    For j := 0 To Dim Do
      If i <> j Then
        Line_X_Sub_V_Mul_Line_Y(j, m[i, j], i);
  End;
End;

Function InvertMatrix2(M: TMatrix3x3): TMatrix3x3;
Var
  det: TBaseType;
Begin
  det := -m[0, 0] * m[1, 2] * m[2, 1]
    - m[1, 0] * m[0, 1] * m[2, 2]
    - m[2, 0] * m[0, 2] * m[1, 1]
    + m[0, 0] * m[1, 1] * m[2, 2]
    + m[1, 0] * m[0, 2] * m[2, 1]
    + m[2, 0] * m[0, 1] * m[1, 2];
  Result[0, 0] := (m[1, 1] * m[2, 2] - m[1, 2] * m[2, 1]) / det;
  Result[0, 1] := (-m[0, 1] * m[2, 2] + m[0, 2] * m[2, 1]) / det;
  Result[0, 2] := (m[0, 1] * m[1, 2] - m[0, 2] * m[1, 1]) / det;
  Result[1, 0] := (-m[1, 0] * m[2, 2] + m[1, 2] * m[2, 0]) / det;
  Result[1, 1] := (m[0, 0] * m[2, 2] - m[0, 2] * m[2, 0]) / det;
  Result[1, 2] := (-m[0, 0] * m[1, 2] + m[0, 2] * m[1, 0]) / det;
  Result[2, 0] := (m[1, 0] * m[2, 1] - m[1, 1] * m[2, 0]) / det;
  Result[2, 1] := (-m[0, 0] * m[2, 1] + m[0, 1] * m[2, 0]) / det;
  Result[2, 2] := (m[0, 0] * m[1, 1] - m[0, 1] * m[1, 0]) / det;
End;


Function TransposeMatrix(Const M: TMatrix3x3): TMatrix3x3;
Var
  i, j: Integer;
Begin
  For i := 0 To 2 Do
    For j := 0 To 2 Do
      result[i, j] := M[j, i];
End;

Function TransposeMatrix(Const M: TMatrix4x4): TMatrix4x4;
Var
  i, j: Integer;
Begin
  For i := 0 To 3 Do
    For j := 0 To 3 Do
      result[i, j] := M[j, i];
End;

Function MulVectorMatrix(Const V: TVector3; Const M: TMatrix3x3): TVector3;
Begin
  Result.x := m[0, 0] * v.x + m[0, 1] * v.y + m[0, 2] * v.z;
  Result.y := m[1, 0] * v.x + m[1, 1] * v.y + m[1, 2] * v.z;
  Result.z := m[2, 0] * v.x + m[2, 1] * v.y + m[2, 2] * v.z;
End;

Function MulVectorMatrix(Const V: TVector4; Const M: TMatrix4x4): TVector4;
Begin
  result.x := m[0, 0] * v.x + m[1, 0] * v.y + m[2, 0] * v.z + m[3, 0] * v.w;
  result.y := m[0, 1] * v.x + m[1, 1] * v.y + m[2, 1] * v.z + m[3, 1] * v.w;
  result.z := m[0, 2] * v.x + m[1, 2] * v.y + m[2, 2] * v.z + m[3, 2] * v.w;
  result.w := m[0, 3] * v.x + m[1, 3] * v.y + m[2, 3] * v.z + m[3, 3] * v.w;
End;

Function MulMatrix(Const m1, m2: TMatrix3x3): TMatrix3x3;
Var
  i, j, k: Integer;
Begin
  // Init Mit 0
  For j := 0 To 2 Do
    For i := 0 To 2 Do
      result[i, j] := 0;
  // Durchführen der "Einfachen" Matrizenmultiplikation in O(n^3)
  For j := 0 To 2 Do
    For i := 0 To 2 Do
      For k := 0 To 2 Do
        result[i, j] := m1[k, j] * m2[i, k] + result[i, j];
End;

Function MulMatrix(Const m1, m2: TMatrix4x4): TMatrix4x4;
Var
  i, j, k: Integer;
Begin
  // Init Mit 0
  For j := 0 To 3 Do
    For i := 0 To 3 Do
      result[i, j] := 0;
  // Durchführen der "Einfachen" Matrizenmultiplikation in O(n^3)
  For j := 0 To 3 Do
    For i := 0 To 3 Do
      For k := 0 To 3 Do
        result[i, j] := m1[k, j] * m2[i, k] + result[i, j];
End;

Function GenerateRotationMatrix(Angle: TBaseType; RotationVector: TVector3): TMatrix4x4;
Var
  OneSubC, s, c: float;
Begin
  RotationVector := NormV3(RotationVector);
  If lenV3(RotationVector) = 0 Then
    Raise exception.Create('"uvectormath.GenerateRotationMatrix" Error invalid "RotationVector".');
  If Angle <> 0 Then Begin
    sincos(degtorad(Angle), s, c);
    OneSubC := 1 - c;
    result[0, 0] := RotationVector.x * RotationVector.x * OneSubC + c;
    result[1, 0] := RotationVector.x * RotationVector.y * OneSubC - RotationVector.z * s;
    result[2, 0] := RotationVector.x * RotationVector.z * OneSubC + RotationVector.y * s;
    result[3, 0] := 0;
    result[0, 1] := RotationVector.y * RotationVector.x * OneSubC + RotationVector.z * s;
    result[1, 1] := RotationVector.y * RotationVector.y * OneSubC + c;
    result[2, 1] := RotationVector.y * RotationVector.z * OneSubC - RotationVector.x * s;
    result[3, 1] := 0;
    result[0, 2] := RotationVector.z * RotationVector.x * OneSubC - RotationVector.y * s;
    result[1, 2] := RotationVector.z * RotationVector.y * OneSubC + RotationVector.x * s;
    result[2, 2] := RotationVector.z * RotationVector.z * OneSubC + c;
    result[3, 2] := 0;
    result[0, 3] := 0;
    result[1, 3] := 0;
    result[2, 3] := 0;
    result[3, 3] := 1;
  End
  Else Begin
    result := IdentityMatrix4x4;
  End;
End;

Function PointArrIsConvexPolygon(Const Points: TVector2Array): Boolean;
Const
  pi2 = pi * 2;

Var
  i, ip1, im1: Integer;
  ai: Double;
  v1, v2: TVector2;
  mode: Shortint; // 1 = alle Winkel müssen >= 180 sein, -1 alle Winkel müssen <= 180 sein
Begin
  // Wenn es nur 2 oder Weniger Punkte
  If High(points) < 2 Then Begin
    result := false;
  End
    // Ein Dreieck
  Else If High(points) = 2 Then Begin
    result := true;
  End
    // Ein Polygon
  Else Begin
    result := true;
    mode := 0;
    For i := 0 To high(Points) Do Begin
      ip1 := (i + 1) Mod (high(points) + 1);
      im1 := (i + high(points)) Mod (high(points) + 1);
      // Ermitteln des Winkels zwischen den Punkten im1, i, ip1
      v1 := SubV2(points[im1], points[i]);
      v2 := SubV2(points[ip1], points[i]);
      ai := arctan2(v2.y, v2.x) - arctan2(v1.y, v1.x);
      While ai > pi2 Do
        ai := ai - pi2;
      While ai < 0 Do
        ai := ai + pi2;
      // Ermitteln des Drehsinnes des Polygones
      If mode = 0 Then Begin
        If ai <> pi Then Begin
          If ai > pi Then
            mode := 1
          Else
            Mode := -1;
        End;
      End
      Else Begin
        // überprüfen, ob die Winkel dem Drehsinn entsprechen
        If (Mode = 1) And (ai < pi) Then Begin
          result := false;
          exit;
        End;
        If (Mode = -1) And (ai > pi) Then Begin
          result := false;
          exit;
        End;
      End;
    End;
  End;
End;

(* Implemenentiert ist der Jarvis' Wrap Algorithmus *)

Function PointsToConvexHull(Const Points: TVector2Array): TVector2Array;
// less_xy_2(p, q)
//   <=> p is lexicographically smaller than q
  Function Less_xy_2(p, q: TVector2): Boolean;
  Begin
    result := (p.x < q.x) Or ((p.x = q.x) And (p.y < q.y));
  End;

  // rightturn_2(p, q, r)
  //   <=> p is right of all directed lines from q to r
  Function rightturn_2(p, q, r: TVector2): Boolean;
  Begin
    result := (q.x - p.x) * (r.y - p.y) < (q.y - p.y) * (r.x - p.x);
  End;

Var
  q_next, q_now, p_start, h, i: Integer;
Begin
  setlength(result, high(Points) + 1);
  If High(result) > 2 Then Begin
    // find point with smallest x-coordinate
    p_start := 0;
    For i := 1 To high(Points) Do Begin
      If Less_xy_2(Points[i], Points[p_start]) Then Begin
        p_start := i;
      End;
    End;
    // set q_next to some OTHER point
    For i := 0 To high(Points) Do
      If i <> p_start Then Begin
        q_next := i;
        break;
      End;
    // wrapping
    h := 0;
    q_now := p_start;
    Repeat
      result[h] := Points[q_now];
      inc(h);
      For i := 0 To high(Points) Do
        If rightturn_2(Points[q_now], Points[q_next], Points[i]) Then
          q_next := i;
      q_now := q_next;
      q_next := p_start;
    Until (p_start = q_now);
    // Beschränken der Ausgabe auf die Hin sortierte Convexe Hülle
    setlength(result, h);
  End
  Else Begin
    // Kopieren wenn wir weniger als 4 Punkte haben
    For i := 0 To high(Points) Do
      result[i] := points[i];
  End;
End;

// Find the area of a polygon using a method based on Green's theorem.
//
// The algebraic sign of the area is positive for counterclockwise
// ordering of vertices in the X-Y plane, and negative for
// clockwise ordering.
//
// Reference:  "Centroid of a Polygon" in Graphics Gems IV,
// by Gerard Bashein and Paul R. Detmer,
// Paul S. Heckbert (editor), Academic Press, 1994, pp. 3-6.

Function CalulatePolygonArea(Const Points: TVector2Array): TBaseType;
Var
  i: Integer;
  sum: TBaseType;
Begin
  If High(Points) < 2 Then Begin
    result := 0;
    exit;
  End;
  sum := 0;
  For i := 0 To High(Points) Do Begin
    sum := sum + Points[i].X * Points[(i + 1) Mod (high(Points) + 1)].Y -
      Points[(i + 1) Mod (high(Points) + 1)].X * Points[i].Y
  End;
  result := 0.5 * abs(sum);
End;

Function CalculateOrthoMatrix(EyePos, Dir, Up: TVector3): TMatrix4x4;
Var
  r, u: TVector3;
  TranslationMatrix: TMatrix4x4;
  i, j: Integer;
Begin
  // Erzeugen der Translationsmatrix
  For i := 0 To 3 Do
    For j := 0 To 3 Do
      If i = j Then
        TranslationMatrix[i, j] := 1
      Else
        TranslationMatrix[i, j] := 0;
  TranslationMatrix[3, 0] := -EyePos.x;
  TranslationMatrix[3, 1] := -EyePos.y;
  TranslationMatrix[3, 2] := -EyePos.z;
  // Berechnen der Basisvektoren
  r := CrossV3(dir, up);
  u := CrossV3(r, dir);
  // Normieren der Basisvektoren
  r := NormV3(r);
  u := NormV3(u);
  dir := Normv3(dir);
  // Erzeugen der Basiswechsel Matrix
  result[0, 0] := r.x;
  result[0, 1] := r.y;
  result[0, 2] := r.z;
  result[0, 3] := 0;
  result[1, 0] := u.x;
  result[1, 1] := u.y;
  result[1, 2] := u.z;
  result[1, 3] := 0;
  result[2, 0] := dir.x;
  result[2, 1] := dir.y;
  result[2, 2] := dir.z;
  result[2, 3] := 0;
  result[3, 0] := 0;
  result[3, 1] := 0;
  result[3, 2] := 0;
  result[3, 3] := 1;
  result := TransposeMatrix(result);
  result := MulMatrix(result, TranslationMatrix);
End;

Procedure GenerateShadowMatrix(Out ShadowMatrix: TMatrix4x4; Const Normal, Point: TVector3; Light: TVector4);
Var
  d, dot: Real;
Begin
  d := -normal.X * point.X - normal.Y * point.Y - normal.Z * point.Z;
  dot := normal.X * light.X + normal.Y * light.Y + normal.Z * light.Z + d * light.W;
  ShadowMatrix[0, 0] := -light.X * normal.X + dot;
  ShadowMatrix[0, 1] := -light.Y * normal.X;
  ShadowMatrix[0, 2] := -light.Z * normal.X;
  ShadowMatrix[0, 3] := -light.W * normal.X;
  ShadowMatrix[1, 0] := -light.X * normal.Y;
  ShadowMatrix[1, 1] := -light.Y * normal.Y + dot;
  ShadowMatrix[1, 2] := -light.Z * normal.Y;
  ShadowMatrix[1, 3] := -light.W * normal.Y;
  ShadowMatrix[2, 0] := -light.X * normal.Z;
  ShadowMatrix[2, 1] := -light.Y * normal.Z;
  ShadowMatrix[2, 2] := -light.Z * normal.Z + dot;
  ShadowMatrix[2, 3] := -light.W * normal.Z;
  ShadowMatrix[3, 0] := -light.X * d;
  ShadowMatrix[3, 1] := -light.Y * d;
  ShadowMatrix[3, 2] := -light.Z * d;
  ShadowMatrix[3, 3] := -light.W * d + dot;
End;

Function ArcTangens(Const x, y: Extended): Extended;
Begin
  Result := 0;
  If (x = 0) Then Begin
    If (Y >= 0) Then result := 90;
    If (Y < 0) Then result := 270;
  End
  Else Begin
    result := radtodeg(arctan(Y / X));
    If ((X < 0) And (y > 0)) Or ((x < 0) And (Y <= 0)) Then result := 180 + result;
    If (X > 0) And (Y < 0) Then result := 360 + result;
  End;
End;

Function PointInRect(Const P, TL, BR: TVector2): Boolean;
Begin
  result :=
    (p.x <= max(tl.x, br.x)) And
    (p.x >= min(tl.x, br.x)) And
    (p.y <= max(tl.y, br.y)) And
    (p.y >= min(tl.y, br.y));
End;

Function PointInCube(Const P, TL, BR: Tvector3): Boolean;
Begin
  result :=
    (p.x <= max(tl.x, br.x)) And
    (p.x >= min(tl.x, br.x)) And
    (p.y <= max(tl.y, br.y)) And
    (p.y >= min(tl.y, br.y)) And
    (p.z <= max(tl.z, br.z)) And
    (p.z >= min(tl.z, br.z));
End;

Function PointInOOBB(Const P, p1, p2, p3, p4: TVector2): Boolean;
Var
  dif2, dif1: TVector2;
Begin
  result := False;
  dif1 := SubV2(p1, p);
  dif2 := SubV2(p1, p2);
  If DotV2(dif1, dif2) < 0 Then
    exit;
  dif1 := SubV2(p2, p);
  dif2 := SubV2(p2, p3);
  If DotV2(dif1, dif2) < 0 Then
    exit;
  dif1 := SubV2(p3, p);
  dif2 := SubV2(p3, p4);
  If DotV2(dif1, dif2) < 0 Then
    exit;
  dif1 := SubV2(p4, p);
  dif2 := SubV2(p4, p1);
  If DotV2(dif1, dif2) < 0 Then
    exit;
  result := True;
End;

Function IntersectLine(Const A, B, C, D: TVector2): Boolean;
Var
  r, s, dett, det, a00, a01, a10, a11, a20, a21: TBaseType;
Begin
  result := false;
  a00 := b.x - a.x;
  a01 := b.y - a.y;
  a10 := -(d.x - c.x);
  a11 := -(d.y - c.y);
  a20 := c.x - a.x;
  a21 := c.y - a.y;
  det := a00 * a11 - a01 * a10;
  If det <> 0 Then Begin
    dett := a20 * a11 - a21 * a10;
    r := dett / det;
    dett := a00 * a21 - a01 * a20;
    s := dett / det;
    result := (r >= 0) And (r <= 1) And (s >= 0) And (s <= 1);
  End
  Else Begin
    result := false;
  End;
End;

Function IntersectOOBB(Const A, B, C, D, AA, BB, CC, DD: TVector2): Boolean;
Begin
  result := PointInOOBB(a, aa, bb, cc, dd) Or
    PointInOOBB(b, aa, bb, cc, dd) Or
    PointInOOBB(c, aa, bb, cc, dd) Or
    PointInOOBB(d, aa, bb, cc, dd) Or
    PointInOOBB(AA, a, b, c, d) Or
    PointInOOBB(BB, a, b, c, d) Or
    PointInOOBB(CC, a, b, c, d) Or
    PointInOOBB(DD, a, b, c, d);
End;

Function InertialTensor(v: TVector3; m: Single): TMatrix3x3;
Begin
  Result := M3x3(
    -m * (-sqr(v.z) - sqr(v.y)), -m * (v.y * v.x), -m * (v.z * v.x),
    -m * (v.y * v.x), -m * (-sqr(v.z) - sqr(v.x)), -m * (v.z * v.y),
    -m * (v.z * v.x), -m * (v.z * v.y), -m * (-sqr(v.y) - sqr(v.x)));
End;

Function GetNormal(EbenenPunkt1, EbenenPunkt2, EbenenPunkt3, Punktinnen: Tvector3): Tvector3;
Var
  erg, d1, d2: Tvector3;
Begin
  d1 := subv3(EbenenPunkt1, EbenenPunkt2); // Berechnen der Richtungsvektoren der Ebene
  d2 := subv3(EbenenPunkt1, EbenenPunkt3); // Berechnen der Richtungsvektoren der Ebene
  erg := CrossV3(d1, d2); // Berechnen eines Senkrecht auf der Ebene Stehenden Vektors
  erg := normV3(erg); // Normieren des Vektors
  // Schaun ob unser Normalenvektor auf den Halbraum zeigt in dem unser Punktinnen liegt
  d1 := subv3(EbenenPunkt1, Punktinnen);
  If DotV3(erg, d1) < 0 Then // Wenn nicht dann mus er umgedreht werden
    erg := v3(-erg.x, -erg.y, -erg.z);
  result := erg;
End;

Function Print(Const V: Tvector2): String; overload;
Begin
  result := '(' + floattostr(v.x) + '/' + floattostr(v.y) + ')';
End;

Function Print(Const V: Tvector3): String; overload;
Begin
  result := '(' + floattostr(v.x) + '/' + floattostr(v.y) + '/' + floattostr(v.z) + ')';
End;

Function Print(Const V: Tvector4): String; overload;
Begin
  result := '(' + floattostr(v.x) + '/' + floattostr(v.y) + '/' + floattostr(v.z) + '/' + floattostr(v.w) + ')';
End;

Function Print(Const M: TMatrix3x3): String; overload;
Var
  i, j: integer;
Begin
  result := '';
  For j := 0 To 2 Do Begin
    result := result + '(';
    For i := 0 To 2 Do Begin
      result := result + floattostr(m[i, j]);
      If i <> 2 Then
        result := result + '/'
      Else
        result := result + ')'
    End;
    If j <> 2 Then result := result + #13;
  End;
End;

Function Print(Const M: TMatrix4x4): String; overload;
Var
  i, j: integer;
Begin
  result := '';
  For j := 0 To 3 Do Begin
    result := result + '(';
    For i := 0 To 3 Do Begin
      result := result + floattostr(m[i, j]);
      If i <> 3 Then
        result := result + '/'
      Else
        result := result + ')'
    End;
    If j <> 3 Then result := result + #13;
  End;
End;

End.

